import json
import logging
import sys
from typing import Optional, Dict, Any, List

# NOUVEL IMPORT pour la nouvelle bibliothèque
from google import genai
# Les 'types' sont maintenant accessibles via genai.types (ex: genai.types.GenerateContentConfig)

import mistralai
from mistralai.client import MistralClient

from fastapi import HTTPException, status
import anyio
import asyncio # Ajout pour asyncio.sleep

from core.config import settings
from services.romeo_service import romeo_service 

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# --- DEBUGGING PRINTS ---
try:
    logger.info(f"Loaded mistralai package from: {mistralai.__file__}")
    logger.info(f"mistralai package version: {mistralai.__version__}")
    if hasattr(MistralClient, '__module__'):
        logger.info(f"MistralClient class module: {MistralClient.__module__}")
        client_module = sys.modules.get(MistralClient.__module__)
        if client_module and hasattr(client_module, '__file__'):
            logger.info(f"MistralClient class file: {client_module.__file__}")
except Exception as e:
    logger.error(f"Error during mistralai debug info collection: {e}")

class AIService:
    def __init__(self):
        self.provider = settings.LLM_PROVIDER
        self.model_name = settings.GEMINI_MODEL_NAME if self.provider == "gemini" else settings.MISTRAL_MODEL_NAME

        # self.client sera maintenant le client générique Mistral ou le client Google Gen AI
        self.client = None 
        # Pour Google Gen AI, nous aurons un client spécifique pour les appels
        self.gemini_genai_client = None 

        if self.provider == "gemini":
            if not settings.GEMINI_API_KEY:
                logger.warning("GEMINI_API_KEY n'est pas configurée. Le service Gemini ne sera pas disponible.")
                self.provider = None # Désactiver Gemini si la clé n'est pas là
            else:
                # ANCIEN : genai.configure(api_key=settings.GEMINI_API_KEY)
                # ANCIEN : self.client = genai.GenerativeModel(self.model_name)

                # NOUVEAU : Initialise le client directement (la clé API peut être passée ici)
                self.gemini_genai_client = genai.Client(api_key=settings.GEMINI_API_KEY)
                logger.info(f"Client Google Gen AI configuré.")
        elif self.provider == "mistral":
            if not settings.MISTRAL_API_KEY:
                logger.warning("MISTRAL_API_KEY n'est pas configurée. Le service Mistral ne sera pas disponible.")
                self.provider = None 
            else:
                self.client = MistralClient(api_key=settings.MISTRAL_API_KEY)
                logger.info(f"Client Mistral configuré avec le modèle: {self.model_name}")
        else:
            logger.warning(f"Fournisseur LLM '{self.provider}' non supporté ou non configuré.")

    async def _call_gemini_api(self, prompt: str) -> Any:
        # ANCIEN : if not self.client:
        # NOUVEAU :
        if not self.gemini_genai_client:
            raise RuntimeError("Client Google Gen AI non initialisé. Vérifiez votre configuration de clé API.")

        # Configuration de génération pour forcer la sortie JSON
        # ANCIEN : generation_config = types.GenerationConfig(
        # NOUVEAU : Utilise genai.types pour les classes de configuration et de sécurité
        generation_config = genai.types.GenerateContentConfig( 
            temperature=0.7, 
            top_p=0.95,      
            top_k=0,         
            max_output_tokens=4096, 
            response_mime_type="application/json", 
        )

        # Configuration de sécurité pour gérer le contenu potentiellement bloqué.
        safety_settings = {
            genai.types.HarmCategory.HARM_CATEGORY_HARASSMENT: genai.types.HarmBlockThreshold.BLOCK_NONE,
            genai.types.HarmCategory.HARM_CATEGORY_HATE_SPEECH: genai.types.HarmBlockThreshold.BLOCK_NONE,
            genai.types.HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT: genai.types.HarmBlockThreshold.BLOCK_NONE,
            genai.types.HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT: genai.types.HarmBlockThreshold.BLOCK_NONE,
        }

        try:
            logger.info("Appel à l'API Gemini (nouvel SDK)...")
            # ANCIEN : response = await self.client.generate_content_async(
            # ANCIEN :     prompt,
            # ANCIEN :     generation_config=generation_config,
            # ANCIEN :     safety_settings=safety_settings
            # NOUVEAU : Appel sur le client, et le nom du modèle est passé à chaque appel
            response = await self.gemini_genai_client.generate_content(
                model=self.model_name, # Le nom du modèle est passé ici
                contents=prompt,
                config=generation_config, # Le paramètre est 'config' maintenant
                safety_settings=safety_settings
            )
            # Accédez au texte généré
            return response.text
        except Exception as e:
            logger.error(f"Erreur lors de l'appel à l'API Gemini (nouvel SDK): {e}")
            raise

    async def _call_mistral_api(self, prompt: str) -> Any:
        if not self.client:
            raise RuntimeError("Client Mistral non initialisé. Vérifiez votre configuration de clé API.")

        try:
            logger.info("Appel à l'API Mistral...")
            response = self.client.chat(
                model=self.model_name,
                response_format={"type": "json_object"}, 
                messages=[
                    {"role": "user", "content": prompt}
                ]
            )
            return response.choices[0].message.content
        except Exception as e:
            logger.error(f"Erreur lors de l'appel à l'API Mistral: {e}")
            raise

    async def analyze_job_offer_and_cv(self, job_offer_text: str, cv_text: str) -> dict[str, Any]:
        prompt = f"""
        En tant qu'expert en recrutement, j'ai besoin d'une analyse comparative détaillée entre une offre d'emploi et un CV.
        L'analyse doit identifier les correspondances, les lacunes et les suggestions d'amélioration.

        --- Offre d'emploi ---
        {job_offer_text}

        --- CV ---
        {cv_text}

        La réponse doit être STRICTEMENT au format JSON. Ne pas inclure de texte supplémentaire, de commentaires, ou d'explications avant ou après le bloc JSON.
        Le JSON doit correspondre *exactement* au schéma suivant, en utilisant des types natifs Python (int, str, list, dict) pour les annotations de type :

        ```json
        {{
            "match_score": 0,
            "correspondances": [
                {{
                    "categorie": "Catégorie de correspondance (ex: Compétences techniques)",
                    "elements": ["Élément correspondant 1", "Élément correspondant 2"]
                }}
            ],
            "lacunes": [
                {{
                    "categorie": "Catégorie de lacune (ex: Compétences manquantes)",
                    "elements": ["Élément manquant 1", "Élément manquant 2"]
                }}
            ],
            "suggestions_amelioration_cv": [
                "Suggestion 1 pour améliorer le CV",
                "Suggestion 2 pour améliorer le CV"
            ],
            "suggestions_amelioration_lm": [
                "Suggestion 1 pour améliorer la lettre de motivation",
                "Suggestion 2 pour améliorer la lettre de motivation"
            ],
            "questions_entretien_potentielles": [
                "Question d'entretien potentielle 1",
                "Question d'entretien potentielle 2"
            ],
            "mots_cles_pertinents": [
                "Mot-clé pertinent 1",
                "Mot-clé pertinent 2"
            ],
            "analyse_detaillee": "Analyse textuelle détaillée ici, expliquant le score, les points forts et les faiblesses."
        }}
        ```
        Assurez-vous que toutes les clés et leurs types de valeurs sont respectés. Les listes doivent être vides si aucun élément ne correspond. La chaîne "analyse_detaillee" doit fournir un résumé clair et concis.
        """

        max_retries = 3
        for attempt in range(max_retries):
            try:
                logger.info(f"Envoi du prompt au LLM ({self.provider}) (Tentative {attempt + 1}/{max_retries}).")
                if self.provider == "gemini":
                    llm_response_content = await self._call_gemini_api(prompt)
                elif self.provider == "mistral":
                    llm_response_content = await self._call_mistral_api(prompt)
                else:
                    raise ValueError(f"Fournisseur LLM non supporté: {self.provider}")

                # Extraction robuste du JSON
                json_str = llm_response_content.strip()

                # Supprime les marqueurs de bloc de code si présents
                if json_str.startswith("```json"):
                    start_index = json_str.find('{')
                    end_index = json_str.rfind('}')
                    if start_index != -1 and end_index != -1 and end_index > start_index:
                        json_str = json_str[start_index : end_index + 1]
                    else:
                        json_str = json_str.lstrip("```json").rstrip("```").strip()

                analysis_result = json.loads(json_str)
                logger.info("Analyse LLM terminée avec succès.")

                # Intégration avec le service Romeo
                try:
                    logger.info("Appel au service Romeo pour les prédictions métiers et compétences...")
                    romeo_metiers_predictions = await romeo_service.predict_metiers(job_offer_text)
                    romeo_competences_predictions = await romeo_service.predict_competences(job_offer_text)

                    predicted_rome_metiers = [
                        m["codeRome"] for m in romeo_metiers_predictions if "codeRome" in m
                    ]
                    predicted_rome_competences = [
                        c["codeAppellation"] for c in romeo_competences_predictions if "codeAppellation" in c
                    ]

                    analysis_result["metiers_rome_suggeres_offre"] = list(set(predicted_rome_metiers))
                    analysis_result["competences_rome_suggeres_offre"] = list(set(predicted_rome_competences))

                    logger.info("Intégration Romeo terminée avec succès.")

                except Exception as e:
                    logger.error(f"Erreur lors de l'intégration avec le service Romeo: {e}")
                    analysis_result["metiers_rome_suggeres_offre"] = []
                    analysis_result["competences_rome_suggeres_offre"] = []

                return analysis_result

            except json.JSONDecodeError as e:
                logger.error(f"Erreur de parsing JSON depuis la réponse du LLM (Tentative {attempt + 1}/{max_retries}): {e}.")
                logger.error(f"Réponse brute ayant causé l'erreur:\n---\n{json_str}\n---")
                if attempt < max_retries - 1:
                    logger.info(f"Nouvelle tentative dans {2 ** attempt} secondes...")
                    await asyncio.sleep(2 ** attempt) 
                    continue
                else:
                    raise HTTPException(
                        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                        detail=f"Erreur de format de réponse du LLM après {max_retries} tentatives. Impossible de parser le JSON."
                    )
            except Exception as e:
                logger.error(f"Erreur inattendue lors de l'appel LLM ou traitement (Tentative {attempt + 1}/{max_retries}): {e}")
                if attempt < max_retries - 1:
                    logger.info(f"Nouvelle tentative dans {2 ** attempt} secondes...")
                    await asyncio.sleep(2 ** attempt) 
                    continue
                else:
                    raise HTTPException(
                        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                        detail=f"Erreur inattendue lors de l'analyse LLM après {max_retries} tentatives: {e}"
                    )

# Instanciation unique du service AI
ai_service = AIService()