import asyncio
import os
import sys
import logging
import traceback # Importation de traceback pour afficher les erreurs complètes

# --- PREMIÈRE VÉRIFICATION RAPIDE ---
print("--- Démarrage du script test_romeo_integration.py ---")

# Configure le logging de base pour voir les messages
logging.basicConfig(level=logging.INFO, stream=sys.stdout, format='%(asctime)s - %(levelname)s - %(name)s - %(message)s')
logger = logging.getLogger(__name__)

# --- Configuration du chemin pour les imports ---
# Assurez-vous que le répertoire 'backend/' est dans le PYTHONPATH
# Le script doit être exécuté depuis le répertoire 'backend/'
current_dir = os.path.dirname(os.path.abspath(__file__))
project_root = os.path.abspath(os.path.join(current_dir, '..')) # Ceci devrait être la racine de votre projet (dossier contenant 'backend' et 'frontend')

if project_root not in sys.path:
    sys.path.insert(0, project_root)
    logger.info(f"Ajout de '{project_root}' au sys.path.")

# Vérifiez si le chemin 'backend' est accessible dans sys.path
backend_path = os.path.abspath(current_dir)
if backend_path not in sys.path:
    sys.path.insert(0, backend_path)
    logger.info(f"Ajout de '{backend_path}' au sys.path.")

logger.info(f"sys.path actuel: {sys.path}")

# --- Importe les services nécessaires ---
try:
    # Tente d'importer les modules depuis 'backend.core', 'backend.services', etc.
    # Ceci suppose que le script est exécuté depuis le dossier 'backend'
    from core.config import settings
    from services.romeo_service import romeo_service
    from services.ai_service import ai_service
    
    logger.info("Modules services et settings importés avec succès.")
except ImportError as e:
    logger.error(f"FATAL: Erreur d'importation d'un module. Assurez-vous d'exécuter ce script depuis le répertoire 'backend/' et que toutes les dépendances sont installées.")
    logger.error(f"Erreur: {e}")
    traceback.print_exc() # Affiche la trace complète de l'erreur
    sys.exit(1) # Quitte le script avec un code d'erreur

async def run_romeo_test():
    logger.info("Démarrage du test d'intégration Romeo...")

    # Vérifiez que les clés API et URLs sont configurées
    missing_env_vars = []
    if not settings.FRANCE_TRAVAIL_CLIENT_ID: missing_env_vars.append("FRANCE_TRAVAIL_CLIENT_ID")
    if not settings.FRANCE_TRAVAIL_CLIENT_SECRET: missing_env_vars.append("FRANCE_TRAVAIL_CLIENT_SECRET")
    if not settings.FRANCE_TRAVAIL_TOKEN_URL: missing_env_vars.append("FRANCE_TRAVAIL_TOKEN_URL")
    if not settings.FRANCE_TRAVAIL_ROMEO_API_URL: missing_env_vars.append("FRANCE_TRAVAIL_ROMEO_API_URL")

    if missing_env_vars:
        logger.error(f"Les variables d'environnement suivantes pour France Travail ne sont pas configurées dans votre .env ou ne sont pas chargées: {', '.join(missing_env_vars)}")
        logger.error("Veuillez vérifier votre fichier .env et vous assurer qu'il est accessible et correctement renseigné.")
        return

    if settings.LLM_PROVIDER == "gemini" and not settings.GEMINI_API_KEY:
        logger.error("La clé API GEMINI_API_KEY n'est pas configurée pour le fournisseur Gemini, tel que spécifié par LLM_PROVIDER='gemini'.")
        return
    elif settings.LLM_PROVIDER == "mistral" and not settings.MISTRAL_API_KEY:
        logger.error("La clé API MISTRAL_API_KEY n'est pas configurée pour le fournisseur Mistral, tel que spécifié par LLM_PROVIDER='mistral'.")
        return
    
    logger.info("Toutes les variables d'environnement nécessaires semblent configurées.")

    # Exemple de texte d'offre d'emploi et de CV
    # Adaptez ces textes pour qu'ils soient pertinents pour les prédictions ROME
    job_offer_text = """
    Nous recherchons un développeur Full Stack expérimenté avec une solide maîtrise de Python (Django/FastAPI) et JavaScript (React/Vue.js). 
    Le candidat idéal aura de l'expérience avec les bases de données PostgreSQL, Docker et les principes CI/CD. 
    Des compétences en gestion de projet Agile seraient un plus.
    """

    cv_text = """
    Développeur logiciel avec 5 ans d'expérience dans la création d'applications web. 
    Expertise en Python (FastAPI, Flask) et JavaScript (React). 
    Familiarisé avec PostgreSQL et les déploiements Docker. 
    Expérience en méthodologies Agiles et intégration continue.
    """

    logger.info("Appel à ai_service.analyze_job_offer_and_cv...")
    try:
        analysis_result = await ai_service.analyze_job_offer_and_cv(
            job_offer_text=job_offer_text,
            cv_text=cv_text
        )

        logger.info("\n--- Résultats de l'analyse ---")
        logger.info(f"Score de correspondance: {analysis_result.get('match_score')}")
        
        metiers_rome = analysis_result.get('metiers_rome_suggeres_offre', [])
        if metiers_rome:
            logger.info("Métiers ROME suggérés par l'offre:")
            for metier in metiers_rome:
                logger.info(f"- {metier}")
        else:
            logger.info("Aucun métier ROME suggéré ou l'API Romeo n'a pas retourné de résultats.")

        competences_rome = analysis_result.get('competences_rome_suggeres_offre', [])
        if competences_rome:
            logger.info("Compétences ROME suggérées par l'offre:")
            for competence in competences_rome:
                logger.info(f"- {competence}")
        else:
            logger.info("Aucune compétence ROME suggérée ou l'API Romeo n'a pas retourné de résultats.")
        
        logger.info("\nAnalyse détaillée de l'IA (extrait):")
        logger.info(analysis_result.get('analyse_detaillee', '')[:500] + "...") # Affiche les 500 premiers caractères

    except Exception as e:
        logger.error(f"Une erreur est survenue lors de l'analyse via ai_service: {e}")
        traceback.print_exc() # Affiche la trace complète de l'erreur

    print("--- Fin du script test_romeo_integration.py ---")

if __name__ == "__main__":
    asyncio.run(run_romeo_test())